@@ -499,12 +499,18 @@ class Linter:
                             cls._report_conflicting_fixes_same_anchor(message)
                             for lint_result in linting_errors:
                                 lint_result.fixes = []
-                        elif fixes == last_fixes:  # pragma: no cover
+                        elif fixes == last_fixes:
                             # If we generate the same fixes two times in a row,
                             # that means we're in a loop, and we want to stop.
                             # (Fixes should address issues, hence different
                             # and/or fewer fixes next time.)
-                            cls._warn_unfixable(crawler.code)
+                            # This is most likely because fixes could not be safely
+                            # applied last time, so we should stop gracefully.
+                            linter_logger.debug(
+                                f"Fixes generated for {crawler.code} are the same as "
+                                "the previous pass. Assuming that we cannot apply them "
+                                "safely. Passing gracefully."
+                            )
                         else:
                             # This is the happy path. We have fixes, now we want to
                             # apply them.
@@ -514,7 +520,9 @@ class Linter:
                                 config.get("dialect_obj"),
                                 crawler.code,
                                 anchor_info,
+                                fix_even_unparsable=config.get("fix_even_unparsable"),
                             )
+
                             # Check for infinite loops. We use a combination of the
                             # fixed templated file and the list of source fixes to
                             # apply.
@@ -522,7 +530,14 @@ class Linter:
                                 new_tree.raw,
                                 tuple(new_tree.source_fixes),
                             )
-                            if not _valid:
+                            # Was anything actually applied? If not, then the fixes we
+                            # had cannot be safely applied and we should stop trying.
+                            if loop_check_tuple == (tree.raw, tuple(tree.source_fixes)):
+                                linter_logger.debug(
+                                    f"Fixes for {crawler.code} could not be safely be "
+                                    "applied. Likely due to initially unparsable file."
+                                )
+                            elif not _valid:
                                 # The fixes result in an invalid file. Don't apply
                                 # the fix and skip onward. Show a warning.
                                 linter_logger.warning(
