@@ -19,6 +19,7 @@ from sqlfluff.core.parser import (
     Matchable,
     OneOf,
     Ref,
+    RegexLexer,
     RegexParser,
     Sequence,
     StringLexer,
@@ -60,7 +61,17 @@ databricks_dialect.insert_lexer_matchers(
     before="equals",
 )
 
+databricks_dialect.insert_lexer_matchers(
+    # Notebook Cell Delimiter:
+    # https://learn.microsoft.com/en-us/azure/databricks/notebooks/notebook-export-import#sql-1
+    [
+        RegexLexer("command", r"(\r?\n){2}-- COMMAND ----------(\r?\n)", CodeSegment),
+    ],
+    before="newline",
+)
+
 databricks_dialect.add(
+    CommandCellSegment=TypedParser("command", CodeSegment, type="statement_terminator"),
     DoubleQuotedUDFBody=TypedParser(
         "double_quote",
         CodeSegment,
@@ -189,9 +200,22 @@ databricks_dialect.add(
         IdentifierSegment,
         type="properties_naked_identifier",
     ),
+    LocationWithCredentialGrammar=Sequence(
+        "LOCATION",
+        Ref("QuotedLiteralSegment"),
+        Sequence(
+            "WITH",
+            Bracketed(
+                "CREDENTIAL",
+                Ref("PrincipalIdentifierSegment"),
+            ),
+            optional=True,
+        ),
+    ),
 )
 
 databricks_dialect.replace(
+    DelimiterGrammar=OneOf(Ref("SemicolonSegment"), Ref("CommandCellSegment")),
     # https://docs.databricks.com/en/sql/language-manual/sql-ref-syntax-aux-describe-volume.html
     DescribeObjectGrammar=sparksql_dialect.get_grammar("DescribeObjectGrammar").copy(
         insert=[
@@ -314,6 +338,10 @@ databricks_dialect.replace(
             )
         ],
     ),
+    NotNullGrammar=Sequence(
+        "NOT",
+        "NULL",
+    ),
 )
 
 
@@ -1128,6 +1156,151 @@ class GroupByClauseSegment(sparksql.GroupByClauseSegment):
     )
 
 
+class ColumnConstraintSegment(ansi.ColumnConstraintSegment):
+    """A column constraint, e.g. for CREATE TABLE or ALTER TABLE.
+
+    https://docs.databricks.com/en/sql/language-manual/sql-ref-syntax-ddl-create-table-constraint.html
+    """
+
+    match_grammar = Sequence(
+        Ref("NotNullGrammar", optional=True),
+        Sequence(
+            Sequence(
+                "CONSTRAINT",
+                Ref("ObjectReferenceSegment"),
+                optional=True,
+            ),
+            OneOf(
+                Sequence(
+                    Ref("PrimaryKeyGrammar"),
+                    Ref("ConstraintOptionGrammar", optional=True),
+                ),
+                Sequence(
+                    Ref("ForeignKeyGrammar", optional=True),
+                    "REFERENCES",
+                    Ref("TableReferenceSegment"),
+                    Ref("BracketedColumnReferenceListGrammar", optional=True),
+                    OneOf(
+                        Ref("ForeignKeyOptionGrammar"),
+                        Ref("ConstraintOptionGrammar"),
+                        optional=True,
+                    ),
+                ),
+            ),
+            optional=True,
+        ),
+    )
+
+
+class CreateTableUsingStatementSegment(sparksql.CreateTableStatementSegment):
+    """A `CREATE TABLE [USING]` statement.
+
+    https://docs.databricks.com/en/sql/language-manual/sql-ref-syntax-ddl-create-table-using.html
+    """
+
+    type = "create_table_using_statement"
+
+    match_grammar = Sequence(
+        OneOf(
+            Sequence(
+                Sequence(
+                    "CREATE",
+                    "OR",
+                    optional=True,
+                ),
+                "REPLACE",
+                "TABLE",
+            ),
+            Sequence(
+                "CREATE",
+                Ref.keyword("EXTERNAL", optional=True),
+                "TABLE",
+                Ref("IfNotExistsGrammar", optional=True),
+            ),
+        ),
+        Ref("TableReferenceSegment"),
+        Ref("TableSpecificationSegment", optional=True),
+        Sequence(
+            "USING",
+            Ref("DataSourceSegment"),
+            optional=True,
+        ),
+        AnyNumberOf(Ref("TableClausesSegment")),
+        Sequence(
+            "AS",
+            OneOf(
+                Ref("SelectStatementSegment"),
+                Ref("ValuesClauseSegment"),
+            ),
+            optional=True,
+        ),
+    )
+
+
+class TableSpecificationSegment(BaseSegment):
+    """A table specification, e.g. for CREATE TABLE or ALTER TABLE.
+
+    https://docs.databricks.com/en/sql/language-manual/sql-ref-syntax-ddl-create-table-spec.html
+    """
+
+    type = "table_specification_segment"
+
+    match_grammar = Bracketed(
+        Delimited(
+            Sequence(
+                Ref("ColumnReferenceSegment"),
+                Ref("DatatypeSegment"),
+                AnyNumberOf(
+                    Ref("ColumnPropertiesSegment"),
+                ),
+            ),
+        ),
+    )
+
+
+class ColumnPropertiesSegment(BaseSegment):
+    """Properties for a column in a table specification.
+
+    https://docs.databricks.com/en/sql/language-manual/sql-ref-syntax-ddl-create-table-spec.html
+    """
+
+    type = "column_properties_segment"
+
+    match_grammar = OneOf(
+        Ref("NotNullGrammar"),
+        Ref("GeneratedColumnDefinitionSegment"),
+        Sequence(
+            "DEFAULT",
+            Ref("ColumnConstraintDefaultGrammar"),
+        ),
+        Ref("CommentGrammar"),
+        Ref("ColumnConstraintSegment"),
+        Ref("MaskStatementSegment"),
+    )
+
+
+class TableClausesSegment(BaseSegment):
+    """Clauses for a table specification.
+
+    https://docs.databricks.com/en/sql/language-manual/sql-ref-syntax-ddl-create-table-spec.html
+    """
+
+    type = "table_clauses_segment"
+
+    match_grammar = OneOf(
+        Ref("PartitionClauseSegment"),
+        Ref("ClusterByClauseSegment"),
+        Ref("LocationWithCredentialGrammar"),
+        Ref("OptionsGrammar"),
+        Ref("CommentGrammar"),
+        Ref("TablePropertiesGrammar"),
+        Sequence(
+            "WITH",
+            Ref("RowFilterClauseGrammar"),
+        ),
+    )
+
+
 class GeneratedColumnDefinitionSegment(sparksql.GeneratedColumnDefinitionSegment):
     """A generated column definition, e.g. for CREATE TABLE or ALTER TABLE.
 
@@ -1147,6 +1320,7 @@ class GeneratedColumnDefinitionSegment(sparksql.GeneratedColumnDefinitionSegment
                     OneOf(
                         Ref("FunctionSegment"),
                         Ref("BareFunctionSegment"),
+                        Ref("ExpressionSegment"),
                     ),
                 ),
             ),
@@ -1177,9 +1351,6 @@ class GeneratedColumnDefinitionSegment(sparksql.GeneratedColumnDefinitionSegment
                 ),
             ),
         ),
-        AnyNumberOf(
-            Ref("ColumnConstraintSegment", optional=True),
-        ),
     )
 
 
