@@ -24,8 +24,26 @@ RESERVED_KEYWORDS = [
 
 UNRESERVED_KEYWORDS = [
     "CATALOG",
+    "COMPENSATION",
+    "CRON",
+    "ENFORCED",
+    "EVOLUTION",
+    "FEATURE",
+    "MANAGED",
+    "MASK",
+    "NORELY",
+    "OPTIMIZATION",
     "OPTIMIZE",
-    "ZORDER",
+    "PREDICTIVE",
+    "PROVIDER",
     "PYTHON",
+    "RECIPIENT",
+    "RELY",
+    "SCHEDULE",
     "SQL",
+    "TAGS",
+    "TIMESERIES",
+    "VOLUME",
+    "VOLUMES",
+    "ZORDER",
 ]
