@@ -16,7 +16,6 @@ from sqlfluff.core.parser import (
     Nothing,
     OneOf,
     OptionallyBracketed,
-    ParseMode,
     Ref,
     RegexParser,
     SegmentGenerator,
@@ -32,7 +31,11 @@ from sqlfluff.dialects.dialect_hive_keywords import (
 )
 
 ansi_dialect = load_raw_dialect("ansi")
-hive_dialect = ansi_dialect.copy_as("hive")
+hive_dialect = ansi_dialect.copy_as(
+    "hive",
+    formatted_name="Apache Hive",
+    docstring="The dialect for Apache `Hive <https://hive.apache.org/>`_.",
+)
 
 # Clear ANSI Keywords and add all Hive keywords
 # Commented clearing for now as some are needed for some statements imported
@@ -847,6 +850,22 @@ class MsckTableStatementSegment(BaseSegment):
     )
 
 
+class RowFunctionContentsSegment(BaseSegment):
+    """Row Function Contents."""
+
+    type = "function_contents"
+
+    match_grammar = Sequence(
+        Bracketed(
+            Delimited(
+                Sequence(
+                    Ref("BaseExpressionElementGrammar"),
+                ),
+            ),
+        ),
+    )
+
+
 class FunctionSegment(BaseSegment):
     """A scalar or aggregate function.
 
@@ -865,30 +884,14 @@ class FunctionSegment(BaseSegment):
             # rather than identifiers.
             Sequence(
                 Ref("DatePartFunctionNameSegment"),
-                Bracketed(
-                    Delimited(
-                        Ref("DatetimeUnitSegment"),
-                        Ref(
-                            "FunctionContentsGrammar",
-                            # The brackets might be empty for some functions...
-                            optional=True,
-                        ),
-                    ),
-                    parse_mode=ParseMode.GREEDY,
-                ),
+                Ref("DateTimeFunctionContentsSegment"),
             ),
         ),
         Sequence(
             # This unusual syntax is used to cast the Keyword ROW to
             # to the function_name to avoid rule linting exceptions
             StringParser("ROW", KeywordSegment, type="function_name"),
-            Bracketed(
-                Delimited(
-                    Sequence(
-                        Ref("BaseExpressionElementGrammar"),
-                    ),
-                ),
-            ),
+            Ref("RowFunctionContentsSegment"),
             "AS",
             "ROW",
             Bracketed(
@@ -909,14 +912,7 @@ class FunctionSegment(BaseSegment):
                         Ref("ValuesClauseSegment"),
                     ),
                 ),
-                Bracketed(
-                    Ref(
-                        "FunctionContentsGrammar",
-                        # The brackets might be empty for some functions...
-                        optional=True,
-                    ),
-                    parse_mode=ParseMode.GREEDY,
-                ),
+                Ref("FunctionContentsSegment"),
             ),
             Ref("PostFunctionGrammar", optional=True),
         ),
