@@ -240,6 +240,7 @@ DISPLAY_NAME
 DO
 DOMAIN
 DOUBLE
+DOWNSTREAM
 DYNAMIC
 ECONOMY
 EMAIL
@@ -322,6 +323,7 @@ IMPORT
 IMPORTED
 IMPORTS
 INCLUDE
+INCLUDE_METADATA
 INCLUDE_QUERY_ID
 INDEX
 INFO
@@ -376,6 +378,11 @@ MAX_SIZE
 MAXVALUE
 MEASURES
 MERGE
+METADATA$FILENAME
+METADATA$FILE_ROW_NUMBER
+METADATA$FILE_CONTENT_KEY
+METADATA$FILE_LAST_MODIFIED
+METADATA$START_SCAN_TIME
 MIDDLE_NAME
 MIN_CLUSTER_COUNT
 MINS_TO_BYPASS_MFA
@@ -399,11 +406,13 @@ NOCACHE
 NOCYCLE
 NONE
 NOORDER
+NORELY
 NOTIFICATION
 NOTIFICATION_INTEGRATION
 NOTIFICATION_PROVIDER
 NOTIFY
 NOTIFY_USERS
+NOVALIDATE
 NULL_IF
 NULLS
 OBJECT
@@ -433,8 +442,20 @@ PACKAGES
 PARALLEL
 PARAMETERS
 PARQUET
+PARSE_HEADER
 PARTITION
 PASSWORD
+PASSWORD_HISTORY
+PASSWORD_LOCKOUT_TIME_MINS
+PASSWORD_MAX_AGE_DAYS
+PASSWORD_MAX_LENGTH
+PASSWORD_MAX_RETRIES
+PASSWORD_MIN_AGE_DAYS
+PASSWORD_MIN_LOWER_CASE_CHARS
+PASSWORD_MIN_LENGTH
+PASSWORD_MIN_NUMERIC_CHARS
+PASSWORD_MIN_SPECIAL_CHARS
+PASSWORD_MIN_UPPER_CASE_CHARS
 PAST
 PATTERN
 PER
@@ -474,6 +495,7 @@ REFRESH
 REFRESH_MODE
 REFRESH_ON_CREATE
 REGIONS
+RELY
 REMOVE
 RENAME
 REPEATABLE
@@ -625,6 +647,7 @@ USER_TASK_MANAGED_INITIAL_WAREHOUSE_SIZE
 USER_TASK_TIMEOUT_MS
 USERS
 UTF8
+VALIDATE
 VALIDATE_UTF8
 VALIDATION_MODE
 VALUE
