@@ -1,11 +1,11 @@
 """Implementation of Rule AL05."""
 
+from collections import Counter
 from dataclasses import dataclass, field
 from typing import List, Set, cast
 
-from sqlfluff.core.dialects.base import Dialect
 from sqlfluff.core.dialects.common import AliasInfo
-from sqlfluff.core.parser.segments import BaseSegment
+from sqlfluff.core.parser.segments import BaseSegment, RawSegment
 from sqlfluff.core.rules import (
     BaseRule,
     EvalResultType,
@@ -60,14 +60,19 @@ class Rule_AL05(BaseRule):
     name = "aliasing.unused"
     aliases = ("L025",)
     groups = ("all", "core", "aliasing")
+    config_keywords = ["alias_case_check"]
     crawl_behaviour = SegmentSeekerCrawler({"select_statement"})
     _dialects_requiring_alias_for_values_clause = [
+        "athena",
         "snowflake",
         "tsql",
         "postgres",
     ]
     is_fix_compatible = True
 
+    # config
+    alias_case_check: str
+
     def _eval(self, context: RuleContext) -> EvalResultType:
         violations: List[LintResult] = []
         assert context.segment.is_type("select_statement")
@@ -78,10 +83,12 @@ class Rule_AL05(BaseRule):
 
         # Analyze the SELECT.
         alias: AliasInfo
-        query = AL05Query.from_segment(context.segment, dialect=context.dialect)
-        self._analyze_table_aliases(query, context.dialect)
+        query = cast(
+            AL05Query, AL05Query.from_segment(context.segment, dialect=context.dialect)
+        )
+        self._analyze_table_aliases(query)
 
-        if context.dialect.name == "redshift":
+        if context.dialect.name in ("redshift", "bigquery"):
             # Redshift supports un-nesting using aliases.
             # Detect that situation and ignore.
             # https://docs.aws.amazon.com/redshift/latest/dg/query-super.html#unnest
@@ -92,19 +99,32 @@ class Rule_AL05(BaseRule):
 
             for alias in query.aliases:
                 aliases.add(alias.ref_str)
+                if alias.segment:
+                    aliases.add(self._cs_str_id(alias.segment))
                 if not alias.object_reference:
                     continue  # pragma: no cover
                 for seg in alias.object_reference.segments:
                     if seg.is_type("identifier"):
-                        references.add(seg.raw)
+                        references.add(self._cs_str_id(seg))
 
             # If there's any overlap between aliases and reference
             if aliases.intersection(references):
                 self.logger.debug(
-                    "Overlapping references found. Assuming redshift semi-structured."
+                    "Overlapping references found. Assuming %s semi-structured.",
+                    context.dialect.name,
                 )
                 return None
 
+        # Get the number of times an object (table/view) is referenced. While some
+        # dialects can handle the same table name reference with different schemas,
+        # we don't want to allow a conflict with AL04's uniqueness rule so we grab
+        # the base table name instead of the fully qualified one to determine naming
+        # collisions.
+        ref_counter = Counter(
+            self._cs_str_id(a.object_reference.segments[-1])
+            for a in query.aliases
+            if a.object_reference and a.object_reference.segments
+        )
         for alias in query.aliases:
             # Skip alias if it's required (some dialects require aliases for
             # VALUES clauses).
@@ -112,15 +132,66 @@ class Rule_AL05(BaseRule):
                 alias.from_expression_element, context.dialect.name
             ):
                 continue
+            # Skip alias if the table is referenced more than once, some dialects
+            # require the referenced table names to be unique even if not returned
+            # by the statement.
+            if (
+                alias.object_reference
+                and alias.object_reference.segments
+                and ref_counter.get(
+                    self._cs_str_id(alias.object_reference.segments[-1]),
+                    0,
+                )
+                > 1
+            ):
+                continue
+            # Redshift requires an alias when a `QUALIFY` statement immediately follows
+            # the `FROM` clause.
+            # https://docs.aws.amazon.com/redshift/latest/dg/r_QUALIFY_clause.html
+            if (
+                context.dialect.name == "redshift"
+                and alias.alias_expression
+                and self._followed_by_qualify(context, alias)
+            ):
+                continue
 
-            if alias.aliased and alias.ref_str not in query.tbl_refs:
+            if (
+                alias.aliased
+                and alias.segment
+                and self._cs_str_id(alias.segment) not in query.tbl_refs
+            ):
                 # Unused alias. Report and fix.
                 violations.append(self._report_unused_alias(alias))
         return violations or None
 
-    @classmethod
+    def _cs_str_id(self, identifier: BaseSegment):
+        _normal_val = identifier.raw_normalized(self.alias_case_check == "dialect")
+        if self.alias_case_check == "case_insensitive":
+            _normal_val = _normal_val.upper()
+        elif self.alias_case_check == "quoted_cs_naked_upper":
+            if identifier.is_type("naked_identifier"):
+                _normal_val = _normal_val.upper()
+        elif self.alias_case_check == "quoted_cs_naked_lower":
+            if identifier.is_type("naked_identifier"):
+                _normal_val = _normal_val.lower()
+        return _normal_val
+
+    def _followed_by_qualify(self, context: RuleContext, alias: AliasInfo) -> bool:
+        curr_from_seen = False
+        assert alias.alias_expression
+        for seg in context.segment.segments:
+            if alias.alias_expression.get_end_loc() == seg.get_end_loc():
+                curr_from_seen = True
+            elif curr_from_seen and not seg.is_code:
+                continue
+            elif curr_from_seen and seg.is_type("qualify_clause"):
+                return True
+            elif curr_from_seen:
+                return False
+        return False
+
     def _is_alias_required(
-        cls, from_expression_element: BaseSegment, dialect_name: str
+        self, from_expression_element: BaseSegment, dialect_name: str
     ) -> bool:
         """Given an alias, is it REQUIRED to be present?
 
@@ -138,7 +209,7 @@ class Rule_AL05(BaseRule):
                     # Found a VALUES clause. Is this a dialect that requires
                     # VALUE clauses to be aliased?
                     return (
-                        dialect_name in cls._dialects_requiring_alias_for_values_clause
+                        dialect_name in self._dialects_requiring_alias_for_values_clause
                     )
                 elif any(
                     seg.is_type(
@@ -159,8 +230,7 @@ class Rule_AL05(BaseRule):
         # This should never happen. Return False just to be safe.
         return False  # pragma: no cover
 
-    @classmethod
-    def _analyze_table_aliases(cls, query: AL05Query, dialect: Dialect) -> None:
+    def _analyze_table_aliases(self, query: AL05Query) -> None:
         # Get table aliases defined in query.
         for selectable in query.selectables:
             select_info = selectable.select_info
@@ -171,29 +241,32 @@ class Rule_AL05(BaseRule):
                 # Look at each table reference; if it's an alias reference,
                 # resolve the alias: could be an alias defined in "query"
                 # itself or an "ancestor" query.
-                for r in select_info.reference_buffer:
+                for r in (
+                    select_info.reference_buffer + select_info.table_reference_buffer
+                ):
                     for tr in r.extract_possible_references(
                         level=r.ObjectReferenceLevel.TABLE
                     ):
                         # This function walks up the query's parent stack if necessary.
-                        cls._resolve_and_mark_reference(query, tr.part)
+                        self._resolve_and_mark_reference(
+                            query, cast(RawSegment, tr.segments[0])
+                        )
 
         # Visit children.
         for child in query.children:
-            cls._analyze_table_aliases(cast(AL05Query, child), dialect)
+            self._analyze_table_aliases(cast(AL05Query, child))
 
-    @classmethod
-    def _resolve_and_mark_reference(cls, query: AL05Query, ref: str) -> None:
+    def _resolve_and_mark_reference(self, query: AL05Query, ref: RawSegment) -> None:
         # Does this query define the referenced alias?
-        if any(ref == a.ref_str for a in query.aliases):
+        _ref = self._cs_str_id(ref)
+        if any(_ref == self._cs_str_id(a.segment) for a in query.aliases if a.segment):
             # Yes. Record the reference.
-            query.tbl_refs.add(ref)
+            query.tbl_refs.add(_ref)
         elif query.parent:
             # No. Recursively check the query's parent hierarchy.
-            cls._resolve_and_mark_reference(cast(AL05Query, query.parent), ref)
+            self._resolve_and_mark_reference(query.parent, ref)
 
-    @classmethod
-    def _report_unused_alias(cls, alias: AliasInfo) -> LintResult:
+    def _report_unused_alias(self, alias: AliasInfo) -> LintResult:
         fixes = [LintFix.delete(alias.alias_expression)]  # type: ignore
         # Walk back to remove indents/whitespaces
         to_delete = (
