@@ -63,6 +63,9 @@ class Rule_RF05(BaseRule):
         if dialect_name == "bigquery":
             # In BigQuery, also allow hyphens.
             result.update("-")
+        if dialect_name == "snowflake":
+            # In Snowflake, external stage metadata uses $.
+            result.update("$")
         return "".join(result)
 
     def _eval(self, context: RuleContext) -> Optional[LintResult]:
@@ -101,13 +104,17 @@ class Rule_RF05(BaseRule):
         ):
             return LintResult(memory=context.memory)
 
+        if self._is_aliased_select_clause_element(context):
+            # If selects are aliased, ignore unaliased column reference
+            return None
+
         # Do some extra processing for quoted identifiers.
         if context.segment.is_type("quoted_identifier"):
             # Update the default policy to quoted
             policy = self.quoted_identifiers_policy
 
             # Strip the quotes first
-            identifier = context.segment.raw[1:-1]
+            identifier = context.segment.raw_normalized(casefold=False)
 
             # Skip if in ignore list - repeat check now we've strip the quotes
             if ignore_words_list and identifier.lower() in ignore_words_list:
@@ -215,3 +222,13 @@ class Rule_RF05(BaseRule):
             self.ignore_words_list = []
 
         return self.ignore_words_list
+
+    @staticmethod
+    def _is_aliased_select_clause_element(context: RuleContext) -> bool:
+        for seg in reversed(context.parent_stack):
+            if seg.is_type("alias_expression"):
+                return False
+            if seg.is_type("select_clause_element"):
+                return seg.get_child("alias_expression") is not None
+
+        return False
